#include "elevator.hpp"

Elevator::Elevator ()
	: door_(CLOSED)
	, direction_(IDLE)
	, current_position_()
	, stops_()
	, ELEVATOR_STEP(0.1)
{
	// Intentionally Left Empty
}

bool Elevator::currently_at_stop () const
{
	StopList::const_iterator it;
	Stop current(current_position_, direction_);

	for (it = stops_.begin (); it != stops_.end (); it++)
		if (*it == current)
			return true;

	return false;
}

void Elevator::stop_at (Stop &stop)
{
	StopList::iterator it;

	/* If this is an "ALL" Stop, it supercedes all others */
	if (stop.getDirection() == ALL)
	{
		stops_.remove (stop);
		stops_.push_back (stop);
		return;
	}

	/* Check if this stop already exists. If so, just leave */
	for (it = stops_.begin(); it != stops_.end(); it++)
		if (*it == stop)
			return;

	/* The stop did not already exist, so add it */
	stops_.push_back (stop);
}

float Elevator::distance_from (Position &pos) const
{
	if (current_position_ > pos)
		return current_position_ - pos;

	return pos - current_position_;
}

void Elevator::move ()
{
	static int wait = 0;

	/* Wait around if we need to.
	 *
	 * This is an artificial delay to make the elevators work in a
	 * much more natural fashion. This allows some delay to leave the
	 * doors open, or for the user to press floor selection buttons
	 * before the elevator starts moving
	 */
	if (wait > 0)
	{
		--wait;
		return;
	}

	/* close the door if it is open. This is a requirement to move */
	if (door_ != CLOSED)
	{
		wait = 10;
		close_door ();
		return;
	}

	if (currently_at_stop ())
	{
		wait = 10;									// delay around for 10 steps
		stops_.remove (Stop(current_position_, direction_));
		open_door ();								// call into the GUI to open the door
		return;
	}

	/* Calculate the number of Stops above and below our
	 * current position */
	StopList::const_iterator it;
	Stop current = Stop(current_position_, direction_);
	int stops_above = 0;
	int stops_below = 0;

	for (it = stops_.begin(); it != stops_.begin(); it++)
	{
		if (current < *it)
			++stops_above;

		if (current > *it)
			++stops_below;
	}

	/* Check if we need to change direction */
	if (direction_ == UP && stops_above == 0 && stops_below > 0)
		direction_ = DOWN;

	if (direction_ == DOWN && stops_below == 0 && stops_above > 0)
		direction_ = UP;

	if (stops_above == 0 && stops_below == 0)
		direction_ = IDLE;


	/* Move in the correct direction */
	switch (direction_)
	{
		case IDLE:
			break;
		case UP:
			current_position_ += ELEVATOR_STEP;
			break;
		case DOWN:
			current_position_ -= ELEVATOR_STEP;
			break;
		default:
			std::cout << __FILE__ << ":" << __LINE__ << " Unhandled Elevator Position" << std::endl;
			break;
	}

	/* Call the GUI with our updated position */
	update_position ();
}

bool Elevator::is_idle () const
{
	return direction_ == IDLE;
}

void Elevator::update_position () const
{
	std::cout << "Updating the GUI with our position" << std::endl;
}

void Elevator::open_door () const
{
	std::cout << "Opening Door" << std::endl;
}

void Elevator::close_door () const
{
	std::cout << "Closing Door" << std::endl;
}





/* vim: set ts=4 sts=4 sw=4 noexpandtab textwidth=112: */
